import time
import functools
import logging
from fastutils.funcutils import get_inject_params
from django.http import JsonResponse
from .utils import SimpleJsonEncoder
from .utils import get_request_data
from .pack import SimpleJsonResultPacker

logger = logging.getLogger(__name__)

simple_json_result_packer = SimpleJsonResultPacker()


def apiview(view):
    def wrapper(request, **kwargs):
        package = {}
        try:
            data = get_request_data(request, kwargs)
            params = get_inject_params(view, data)
            result = view(**params)
            package = simple_json_result_packer.pack_result(result)
        except Exception as error:
            logger.exception("Error on getting result in @apiview...")
            package = simple_json_result_packer.pack_error(error)
        return JsonResponse(package, encoder=SimpleJsonEncoder, json_dumps_params={"ensure_ascii": False, "allow_nan": True, "sort_keys": True})
    wrapper.csrf_exempt = True
    return functools.wraps(view)(wrapper)



import inspect
import json
import binascii
import bizerror
from fastutils.jsonutils import SimpleJsonEncoder
from fastutils.typingutils import smart_cast
from django.db import models
from django.core.serializers.json import DjangoJSONEncoder as DjangoJSONEncoderBase
from django.forms.models import model_to_dict
from django.core import serializers

def encode_model(o):
    text = serializers.serialize("json", [o])
    results = json.loads(text)
    obj = results[0]["fields"]
    obj["pk"] = results[0]["pk"]
    return obj

def encode_queryset(q):
    text = serializers.serialize("json", q)
    results = json.loads(text)
    data = []
    for result in results:
        obj = result["fields"]
        obj["pk"] = result["pk"]
        data.append(obj)
    return data

SimpleJsonEncoder.library.register(models.Model, encode_model)
SimpleJsonEncoder.library.register(models.QuerySet, encode_queryset)

def get_request_data(request, extra_view_parameters):
    data = {"_request": request}
    data.update(extra_view_parameters)
    for name, _ in request.GET.items():
        value = request.GET.getlist(name)
        if isinstance(value, (list, tuple, set)) and len(value) == 1:
            data[name] = value[0]
        else:
            data[name] = value
    for name, _ in request.POST.items():
        value = request.POST.getlist(name)
        if isinstance(value, (list, tuple, set)) and len(value) == 1:
            data[name] = value[0]
        else:
            data[name] = value
    if request.body:
        try:
            payload = json.loads(request.body)
            data["_form"] = payload
            data.update(payload)
        except:
            pass
    for name, fobj in request.FILES.items():
        data[name] = fobj
    return data

def get_inject_params(func, data):
    params = {}
    parameters = inspect.signature(func).parameters
    for name, parameter in parameters.items():
        if parameter.default is parameter.empty:
            if not name in data:
                raise bizerror.MissingParameter("Missing required parameter: {0}".format(name))
            value = data[name]
        else:
            value = data.get(name, parameter.default)
        if not parameter.annotation is parameter.empty:
            value = smart_cast(parameter.annotation, value)
        params[name] = value
    return params
